@@ -1,19 +1,19 @@
 /* This file is part of the KDE libraries
  * SPDX-FileCopyrightText: 2009 Dario Freddi <drf at kde.org>
  *
- * SPDX-License-Identifier: LGPL-2.0-only
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 #include "kidletime.h"
 
 #include <config-kidletime.h>
 
-#include "abstractsystempoller.h"
+#include "kabstractidletimepoller_p.h"
 #include "logging.h"
 
 #include <QDir>
-#include <QJsonArray>
 #include <QGuiApplication>
+#include <QJsonArray>
 #include <QPluginLoader>
 #include <QPointer>
 #include <QSet>
@@ -21,7 +21,10 @@
 class KIdleTimeHelper
 {
 public:
-    KIdleTimeHelper() : q(nullptr) {}
+    KIdleTimeHelper()
+        : q(nullptr)
+    {
+    }
     ~KIdleTimeHelper()
     {
         delete q;
@@ -46,15 +49,20 @@ class KIdleTimePrivate
 {
     Q_DECLARE_PUBLIC(KIdleTime)
     KIdleTime *q_ptr;
+
 public:
-    KIdleTimePrivate() : catchResume(false), currentId(0) {}
+    KIdleTimePrivate()
+        : catchResume(false)
+        , currentId(0)
+    {
+    }
 
     void loadSystem();
     void unloadCurrentSystem();
-    void _k_resumingFromIdle();
-    void _k_timeoutReached(int msec);
+    void resumingFromIdle();
+    void timeoutReached(int msec);
 
-    QPointer<AbstractSystemPoller> poller;
+    QPointer<KAbstractIdleTimePoller> poller;
     bool catchResume;
 
     int currentId;
@@ -73,15 +81,18 @@ KIdleTime::KIdleTime()
     Q_D(KIdleTime);
     d->loadSystem();
 
-    connect(d->poller.data(), SIGNAL(resumingFromIdle()), this, SLOT(_k_resumingFromIdle()));
-    connect(d->poller.data(), SIGNAL(timeoutReached(int)), this, SLOT(_k_timeoutReached(int)));
+    connect(d->poller.data(), &KAbstractIdleTimePoller::resumingFromIdle, this, [d]() {
+        d->resumingFromIdle();
+    });
+    connect(d->poller.data(), &KAbstractIdleTimePoller::timeoutReached, this, [d](int msec) {
+        d->timeoutReached(msec);
+    });
 }
 
 KIdleTime::~KIdleTime()
 {
     Q_D(KIdleTime);
     d->unloadCurrentSystem();
-    delete d_ptr;
 }
 
 void KIdleTime::catchNextResumeEvent()
@@ -123,15 +134,20 @@ void KIdleTime::removeIdleTimeout(int identifier)
 {
     Q_D(KIdleTime);
 
-    if (!d->associations.contains(identifier) || !d->poller) {
+    const auto it = d->associations.constFind(identifier);
+    if (it == d->associations.cend() || !d->poller) {
         return;
     }
 
-    int msec = d->associations[identifier];
+    const int msec = it.value();
 
-    d->associations.remove(identifier);
+    d->associations.erase(it);
 
-    if (!d->associations.values().contains(msec)) {
+    const bool isFound = std::any_of(d->associations.cbegin(), d->associations.cend(), [msec](int i) {
+        return i == msec;
+    });
+
+    if (!isFound) {
         d->poller.data()->removeTimeout(msec);
     }
 }
@@ -140,62 +156,96 @@ void KIdleTime::removeAllIdleTimeouts()
 {
     Q_D(KIdleTime);
 
-    QHash< int, int >::iterator i = d->associations.begin();
-    QSet< int > removed;
-    removed.reserve(d->associations.size());
+    std::vector<int> removed;
 
-    while (i != d->associations.end()) {
-        int msec = d->associations[i.key()];
-
-        i = d->associations.erase(i);
-
-        if (!removed.contains(msec) && d->poller) {
+    for (auto it = d->associations.cbegin(); it != d->associations.cend(); ++it) {
+        const int msec = it.value();
+        const bool alreadyIns = std::find(removed.cbegin(), removed.cend(), msec) != removed.cend();
+        if (!alreadyIns && d->poller) {
+            removed.push_back(msec);
             d->poller.data()->removeTimeout(msec);
-            removed.insert(msec);
         }
     }
+
+    d->associations.clear();
 }
 
 static QStringList pluginCandidates()
 {
     QStringList ret;
+
     const QStringList libPath = QCoreApplication::libraryPaths();
-    for (const QString &path : libPath ) {
-        const QDir pluginDir(path + QLatin1String("/kf5/org.kde.kidletime.platforms"));
+    for (const QString &path : libPath) {
+#ifdef Q_OS_MACOS
+        const QDir pluginDir(path + QStringLiteral("/kf6/kidletime"));
+#else
+        const QDir pluginDir(path + QStringLiteral("/kf6/org.kde.kidletime.platforms"));
+#endif
         if (!pluginDir.exists()) {
             continue;
         }
-        for (const QString &entry : pluginDir.entryList(QDir::Files | QDir::NoDotAndDotDot)) {
+
+        const auto entries = pluginDir.entryList(QDir::Files | QDir::NoDotAndDotDot);
+
+        ret.reserve(ret.size() + entries.size());
+        for (const QString &entry : entries) {
             ret << pluginDir.absoluteFilePath(entry);
         }
     }
+
     return ret;
 }
 
-static AbstractSystemPoller *loadPoller()
+static bool checkPlatform(const QJsonObject &metadata, const QString &platformName)
 {
+    const QJsonArray platforms = metadata.value(QStringLiteral("MetaData")).toObject().value(QStringLiteral("platforms")).toArray();
+    return std::any_of(platforms.begin(), platforms.end(), [&platformName](const QJsonValue &value) {
+        return QString::compare(platformName, value.toString(), Qt::CaseInsensitive) == 0;
+    });
+}
+
+static KAbstractIdleTimePoller *loadPoller()
+{
+    const QString platformName = QGuiApplication::platformName();
+
+    const QList<QStaticPlugin> staticPlugins = QPluginLoader::staticPlugins();
+    for (const QStaticPlugin &staticPlugin : staticPlugins) {
+        const QJsonObject metadata = staticPlugin.metaData();
+        if (metadata.value(QLatin1String("IID")) != QLatin1String(KAbstractIdleTimePoller_iid)) {
+            continue;
+        }
+        if (checkPlatform(metadata, platformName)) {
+            auto *poller = qobject_cast<KAbstractIdleTimePoller *>(staticPlugin.instance());
+            if (poller) {
+                if (poller->isAvailable()) {
+                    qCDebug(KIDLETIME) << "Loaded system poller from a static plugin";
+                    return poller;
+                }
+                delete poller;
+            }
+        }
+    }
+
     const QStringList lstPlugins = pluginCandidates();
     for (const QString &candidate : lstPlugins) {
         if (!QLibrary::isLibrary(candidate)) {
             continue;
         }
         QPluginLoader loader(candidate);
-        QJsonObject metaData = loader.metaData();
-        const QJsonArray platforms = metaData.value(QStringLiteral("MetaData")).toObject().value(QStringLiteral("platforms")).toArray();
-        for (auto it = platforms.begin(); it != platforms.end(); ++it) {
-            if (QString::compare(QGuiApplication::platformName(), (*it).toString(), Qt::CaseInsensitive) == 0) {
-                AbstractSystemPoller *poller = qobject_cast< AbstractSystemPoller* >(loader.instance());
-                if (poller) {
-                    qCDebug(KIDLETIME) << "Trying plugin" << candidate;
-                    if (poller->isAvailable()) {
-                        qCDebug(KIDLETIME) << "Using" << candidate << "for platform" << QGuiApplication::platformName();
-                        return poller;
-                    }
-                    delete poller;
+        if (checkPlatform(loader.metaData(), platformName)) {
+            auto *poller = qobject_cast<KAbstractIdleTimePoller *>(loader.instance());
+            if (poller) {
+                qCDebug(KIDLETIME) << "Trying plugin" << candidate;
+                if (poller->isAvailable()) {
+                    qCDebug(KIDLETIME) << "Using" << candidate << "for platform" << platformName;
+                    return poller;
                 }
+                delete poller;
             }
         }
     }
+
+    qCWarning(KIDLETIME) << "Could not find any system poller plugin";
     return nullptr;
 }
 
@@ -224,26 +274,24 @@ void KIdleTimePrivate::unloadCurrentSystem()
     }
 }
 
-void KIdleTimePrivate::_k_resumingFromIdle()
+void KIdleTimePrivate::resumingFromIdle()
 {
     Q_Q(KIdleTime);
 
     if (catchResume) {
-        emit q->resumingFromIdle();
+        Q_EMIT q->resumingFromIdle();
         q->stopCatchingResumeEvent();
     }
 }
 
-void KIdleTimePrivate::_k_timeoutReached(int msec)
+void KIdleTimePrivate::timeoutReached(int msec)
 {
     Q_Q(KIdleTime);
 
-    if (associations.values().contains(msec)) {
-        const auto listKeys = associations.keys(msec);
-        for (int key : listKeys) {
-            emit q->timeoutReached(key);
-            emit q->timeoutReached(key, msec);
-        }
+    const auto listKeys = associations.keys(msec);
+
+    for (const auto key : listKeys) {
+        Q_EMIT q->timeoutReached(key, msec);
     }
 }
 
